@@ -526,7 +526,7 @@ class IP:
     def calc_code_timestamp(self):
         src_path = f"{self.path}/{self.src_path}/"
         self.code_timestamp = common.dir_timestamp(src_path)
-        common.dbg(f"{self.vendor}/{self.name}: calc_code_timestamp={self.code_timestamp}")
+        #common.dbg(f"{self.vendor}/{self.name}: calc_code_timestamp={self.code_timestamp}")
     
     def reset_is_compiled_elaborated(self):
         clean.clean_ip(self, True)
@@ -546,12 +546,12 @@ class IP:
             if self.fresh_code_timestamp == "":
                 src_path = f"{self.path}/{self.src_path}/"
                 self.fresh_code_timestamp = common.dir_timestamp(src_path)
-                common.dbg(f"{self.vendor}/{self.name}: fresh_timestamp={self.fresh_code_timestamp}")
+                #common.dbg(f"{self.vendor}/{self.name}: fresh_timestamp={self.fresh_code_timestamp}")
             if self.fresh_code_timestamp != self.code_timestamp:
                 self.reset_is_compiled_elaborated()
     
     def resolve_dependencies(self):
-        common.dbg(f"Resolving dependencies for IP '{self.vendor}/{self.name}'")
+        #common.dbg(f"Resolving dependencies for IP '{self.vendor}/{self.name}'")
         if self.has_dut:
             if self.dut_ip_type == "fsoc":
                 self.dut_core = get_core(self.dut_fsoc_full_name, True)
@@ -565,7 +565,7 @@ class IP:
         deps_list = []
         self.dependencies.sort(key= get_key)
         for dep in self.dependencies:
-            common.dbg(f"Processing dependency '{dep.vendor}/{dep.target_ip}'")
+            #common.dbg(f"Processing dependency '{dep.vendor}/{dep.target_ip}'")
             deps_deps_list = dep.target_ip_model.get_ordered_deps()
             deps_list += deps_deps_list
             deps_list.append(dep.target_ip_model)
@@ -579,7 +579,7 @@ class IP:
         deps_list = []
         self.dependencies.sort(key= get_key)
         for dep in self.dependencies:
-            common.dbg(f"Processing dependency '{dep.vendor}/{dep.target_ip}'")
+            #common.dbg(f"Processing dependency '{dep.vendor}/{dep.target_ip}'")
             if dep.target_ip_model.is_local:
                 continue
             deps_deps_list = dep.target_ip_model.get_ordered_deps()
@@ -616,7 +616,7 @@ class IP:
     def are_deps_installed(self):
         is_installed = True
         for dep in self.dependencies:
-            common.dbg(f"Checking dependency '{dep.vendor}/{dep.target_ip}' to see if it's installed")
+            #common.dbg(f"Checking dependency '{dep.vendor}/{dep.target_ip}' to see if it's installed")
             dep_ip = get_ip(dep.vendor, dep.target_ip)
             if dep_ip == None:
                 is_installed = False
@@ -666,13 +666,13 @@ class Dependency:
             self.vendor = ""
     
     def resolve(self):
-        common.dbg(f"Dep '{self.vendor}/{self.target_ip}' resolving ...")
+        #common.dbg(f"Dep '{self.vendor}/{self.target_ip}' resolving ...")
         if self.vendor == "":
             self.target_ip_model = get_anon_ip(self.target_ip)
         else:
             self.target_ip_model = get_ip(self.vendor, self.target_ip)
         if self.target_ip_model != None:
-            common.dbg(f"Dep '{self.vendor}/{self.target_ip}' resolved!")
+            #common.dbg(f"Dep '{self.vendor}/{self.target_ip}' resolved!")
             self.vendor = self.target_ip_model.vendor
             self.semver = self.target_ip_model.version
 
@@ -772,12 +772,14 @@ def load_core_cache():
 def check_ip_cache_integrity():
     list = {}
     for vendor in ip_cache:
+        list[vendor] = []
         for ip in ip_cache[vendor]:
             if not ip_cache[vendor][ip].integrity_check():
-                list[vendor] = ip
-                common.warning(f"Removed IP '{vendor}/{ip}' from cache")
+                list[vendor].append(ip)
     for vendor in list:
-        ip_cache[vendor].pop(list[vendor])
+        for ip in list[vendor]:
+            ip_cache[vendor].pop(ip)
+            common.warning(f"Removed IP '{vendor}/{ip}' from cache")
 
 
 def check_core_cache_integrity():
