@@ -11,25 +11,17 @@ from mio import dox
 from mio import results
 from mio import sim
 
+import glob
 import os
 
 
-def main(ip_str, deep_clean):
+def main(ip_str):
     vendor, name = common.parse_dep(ip_str)
     if vendor == "":
         ip = cache.get_anon_ip(name, True)
     else:
         ip = cache.get_ip(vendor, name, True)
-    if deep_clean:
-        common.banner(f"Deep cleaning IP '{ip.vendor}/{ip.name}'")
-        for dep in ip.dependencies:
-            dep_ip = cache.get_ip(dep.vendor, dep.target_ip)
-            if dep_ip == None:
-                common.dbg(f"Cannot find Dependency IP '{dep.vendor}/{dep.target_ip}'")
-            else:
-                clean_ip(dep_ip)
-    else:
-        common.banner(f"Cleaning IP '{ip.vendor}/{ip.name}'")
+    common.banner(f"Cleaning IP '{ip.vendor}/{ip.name}'")
     clean_ip(ip)
     
     if ip.has_dut:
@@ -42,7 +34,9 @@ def main(ip_str, deep_clean):
             dut_ip = cache.get_ip(ip.dut.vendor, ip.dut.target_ip)
             if dut_ip == None:
                 common.fatal(f"Cannot find DUT IP '{ip.dut.vendor}/{ip.dut.target_ip}'")
-            clean_ip(dut_ip)
+            if ip.dut_ip_type != "":
+                if ip.dut_ip_type.target_ip_model.sub_type == "vivado":
+                    clean_ip(dut_ip)
 
 
 def clean_core(core):
@@ -63,22 +57,27 @@ def clean_ip(ip, no_infos=False):
     ip_str = f"{ip.vendor}/{ip.name}"
     ip_dir_name = f"{ip.vendor}__{ip.name}"
     
-    common.remove_dir(cfg.sim_output_dir + '/viv/sim_wd')
-    common.remove_dir(cfg.sim_output_dir + '/vcs/sim_wd')
-    common.remove_dir(cfg.sim_output_dir + '/xcl/sim_wd')
-    common.remove_dir(cfg.sim_output_dir + '/qst/sim_wd')
-    common.remove_dir(cfg.sim_output_dir + '/riv/sim_wd')
+    common.remove_dir(cfg.sim_output_dir + '/viv/sim_wd/' + ip_dir_name)
+    common.remove_dir(cfg.sim_output_dir + '/vcs/sim_wd/' + ip_dir_name)
+    common.remove_dir(cfg.sim_output_dir + '/xcl/sim_wd/' + ip_dir_name)
+    common.remove_dir(cfg.sim_output_dir + '/qst/sim_wd/' + ip_dir_name)
+    common.remove_dir(cfg.sim_output_dir + '/riv/sim_wd/' + ip_dir_name)
     
     common.remove_dir(cfg.sim_output_dir + '/viv/cmp_out/' + ip_dir_name)
     common.remove_dir(cfg.sim_output_dir + '/vcs/cmp_out/' + ip_dir_name)
     common.remove_dir(cfg.sim_output_dir + '/xcl/cmp_out/' + ip_dir_name)
     common.remove_dir(cfg.sim_output_dir + '/qst/cmp_out/' + ip_dir_name)
     
-    #common.remove_dir(cfg.sim_output_dir + '/viv/regr_wd/' + ip_dir_name)
-    #common.remove_dir(cfg.sim_output_dir + '/mdc/regr_wd/' + ip_dir_name)
-    #common.remove_dir(cfg.sim_output_dir + '/vcs/regr_wd/' + ip_dir_name)
-    #common.remove_dir(cfg.sim_output_dir + '/xcl/regr_wd/' + ip_dir_name)
-    #common.remove_dir(cfg.sim_output_dir + '/qst/regr_wd/' + ip_dir_name)
+    for name in glob.glob(cfg.sim_output_dir + f'/viv/regr_wd/{ip_dir_name}__*'):
+        common.remove_dir(name)
+    for name in glob.glob(cfg.sim_output_dir + f'/vcs/regr_wd/{ip_dir_name}__*'):
+        common.remove_dir(name)
+    for name in glob.glob(cfg.sim_output_dir + f'/xcl/regr_wd/{ip_dir_name}__*'):
+        common.remove_dir(name)
+    for name in glob.glob(cfg.sim_output_dir + f'/qst/regr_wd/{ip_dir_name}__*'):
+        common.remove_dir(name)
+    for name in glob.glob(cfg.sim_output_dir + f'/riv/regr_wd/{ip_dir_name}__*'):
+        common.remove_dir(name)
     
     if ip.is_local:
         if no_infos:
