@@ -132,7 +132,8 @@ def main():
         common.exit()
     if cli_args.command == 'clean':
         cache.check_ip_str(cli_args.ip.lower())
-        clean.main(cli_args.ip.lower(), cli_args.deep)
+        #clean.main(cli_args.ip.lower(), cli_args.deep)
+        clean.main(cli_args.ip.lower())
         common.exit()
     if cli_args.command == 'sim':
         sim_job = create_sim_job(cli_args)
@@ -222,7 +223,7 @@ def build_parser():
     
     parser_clean = subparsers.add_parser('clean', help=help_text.clean_help_text, add_help=False)
     parser_clean.add_argument('ip'          , help='Target IP'                                                            )
-    parser_clean.add_argument('-d', "--deep", help='Delete compiled IP external dependencies.', action="store_true", required=False)
+    #parser_clean.add_argument('-d', "--deep", help='Delete compiled IP external dependencies.', action="store_true", required=False)
     
     parser_results = subparsers.add_parser('results', help=help_text.results_help_text, add_help=False)
     parser_results.add_argument('ip'      , help='Target IP'      )
@@ -263,12 +264,16 @@ def create_sim_job(cli_args):
         sim_job.simulator = common.simulators_enum.VIVADO
     common.dbg("Using simulator " + str(sim_job.simulator))
     
-    if not cli_args.C and not cli_args.E and not cli_args.S:
+    if not cli_args.F and not cli_args.C and not cli_args.E and not cli_args.S:
+        sim_job.fsoc      = True
         sim_job.compile   = True
         sim_job.elaborate = True
         sim_job.simulate  = True
     else:
-        sim_job.one_shot = False
+        if cli_args.F:
+            sim_job.fsoc = True
+        else:
+            sim_job.fsoc = False
         if cli_args.C:
             sim_job.compile = True
         else:
@@ -282,9 +287,6 @@ def create_sim_job(cli_args):
         else:
             sim_job.simulate = False
     
-    if sim_job.simulator == common.simulators_enum.VIVADO:
-        sim_job.one_shot = False
-    
     if cli_args.seed == None:
         sim_job.seed = random.randint(1, 2147483646)
         common.dbg("Seed not specified, picked " + str(sim_job.seed))
