@@ -29,9 +29,9 @@ def gen_cov_report(ip_str, is_regression=False, test_suite="", regression_name="
     dir_string = ""
     db_name_string = ""
     if is_regression:
-        cov_path    = cfg.regr_results_dir + f"/{ip.name}_{regression_name}/{regression_timestamp}/cov"
-        merge_path  = cfg.regr_results_dir + f"/{ip.name}_{regression_name}/{regression_timestamp}/cov/merge"
-        report_path = cfg.regr_results_dir + f"/{ip.name}_{regression_name}/{regression_timestamp}/cov/report"
+        cov_path    = cfg.regr_results_dir + f"/{ip.name}__{regression_name}__{regression_timestamp}/cov"
+        merge_path  = cfg.regr_results_dir + f"/{ip.name}__{regression_name}__{regression_timestamp}/cov/merge"
+        report_path = cfg.regr_results_dir + f"/{ip.name}__{regression_name}__{regression_timestamp}/cov/report"
     else:
         cov_path    = cfg.sim_dir + f"/cov"
         merge_path  = cfg.sim_dir + f"/cov/merge/{ip.name}/"
