@@ -3,7 +3,7 @@
 ########################################################################################################################
 
 
-version = "1.0.16"
+version = "1.1.0"
 version_text = f"Moore.io CLI Client v{version}"
 
 
@@ -147,11 +147,9 @@ Usage:
    mio clean IP [OPTIONS]
    
 Options:
-   -d, --deep  Also clean compiled IP dependencies.
    
 Examples:
-   mio clean uvmt_my_ip     # Delete compilation, elaboration and simulation binaries for IP 'uvmt_my_ip'
-   mio clean uvmt_my_ip -d  # Delete compilation, elaboration and simulation binaries for IP 'uvmt_my_ip' and its dependencies"""
+   mio clean uvmt_my_ip  # Delete compilation, elaboration and simulation binaries for IP 'uvmt_my_ip'"""
 
 
 
