@@ -150,7 +150,7 @@ def install_ip_and_deps(vendor, name, global_install, username="", password=""):
     #install_ip_list(ip, deps_to_install, global_install, username, password)
     #cache.scan_and_load_ip_metadata()
     
-    deps_to_install = ip.get_ordered_install_deps()
+    deps_to_install = ip.get_deps_to_install()
     while len(deps_to_install) > 0:
         common.info(f"Installing {len(deps_to_install)} dependencies")
         install_ip_dep_list(ip, deps_to_install, global_install, username, password)
