@@ -543,8 +543,7 @@ def launch_sim_jobs(ip, test_suite, regression, sim_job_list, dry_mode):
         regression_name = f"{cfg.test_suite_name}.{cfg.regression_name}"
         common.banner(f"Running regression '{cfg.regression_name}' from test suite '{cfg.test_suite_name}': {str(len(sim_job_list))} test(s) with {str(regression.max_duration)} hour(s) timeout")
     
-    common.create_dir(cfg.regr_results_dir + "/" + cfg.target_ip_name + "_" + regression_name)
-    common.create_dir(cfg.regr_results_dir + "/" + cfg.target_ip_name + "_" + regression_name + "/" + test_suite.timestamp)
+    common.create_dir(cfg.regr_results_dir + "/" + cfg.target_ip_name + "__" + regression_name + "__" + test_suite.timestamp)
     
     for sim_job in sim_job_list:
         # TEMPORARY UNTIL METRICS REGRESSION SYSTEM IS LIVE
@@ -709,7 +708,7 @@ def print_end_of_regression_msg(ip, results, cov_report_path, test_suite, regres
     else:
         regression_name = cfg.regression_name
     
-    results_path = cfg.regr_results_dir + "/" + ip.name + "_" + regression.name + "/" + test_suite.timestamp
+    results_path = cfg.regr_results_dir + "/" + ip.name + "__" + regression_name + "__" + test_suite.timestamp
     
     if results.passed:
         common.info("************************************************************************************************************************")
