@@ -144,7 +144,7 @@ def main(ip_str, filename="", is_regression=False, test_suite="", regression_nam
     testsuite.set('time', str(total_duration))
     tree = ET.ElementTree(testsuites)
     if is_regression:
-        xml_file_path = cfg.regr_results_dir + "/" + ip.name + "_" + regression_name + "/" + regression_timestamp + "/" + filename + ".xml"
+        xml_file_path = cfg.regr_results_dir + "/" + ip.name + "__" + regression_name + "__" + regression_timestamp + "/" + filename + ".xml"
     else:
         xml_file_path = cfg.sim_dir + "/" + filename + ".xml"
     tree.write(xml_file_path)
@@ -169,7 +169,7 @@ def main(ip_str, filename="", is_regression=False, test_suite="", regression_nam
     
     try:
         if is_regression:
-            html_file_path = cfg.regr_results_dir + "/" + ip.name + "_" + regression_name + "/" + regression_timestamp + "/" + filename + ".html"
+            html_file_path = cfg.regr_results_dir + "/" + ip.name + "__" + regression_name + "__" + regression_timestamp + "/" + filename + ".html"
         else:
             html_file_path = cfg.sim_dir + "/" + filename + ".html"
         with open(html_file_path,'w') as htmlfile:
