@@ -47,7 +47,8 @@ seconds_waited = 0
 num_deps_to_compile = 0
 sem = BoundedSemaphore(1)
 pbar = None
-
+est_time = 0
+overtime = True
 
 
 class StoppableThread(threading.Thread):
@@ -73,7 +74,6 @@ class SimulationJob:
     def __init__(self, ip_str):
         self.vendor, self.ip = common.parse_dep(ip_str)
         self.simulator       = ""
-        self.one_shot        = True
         self.fsoc            = False
         self.compile         = False
         self.elaborate       = False
@@ -146,296 +146,124 @@ def main(sim_job):
     if not doctor.check_simulator_executables(sim_job.simulator):
         common.fatal("Simulator '" + sim_job.simulator + "' not installed properly or environment variable missing")
     
-    if sim_job.simulator == common.simulators_enum.METRICS:
-        eal.init_metrics_workspace()
-    
     convert_cli_args_to_defines (sim_job)
     convert_cli_args_to_plusargs(sim_job)
-    create_sim_directories()
-    
+    create_sim_directories      (sim_job)
     check_dependencies(ip)
-    #ip.update_is_compiled_elaborated(sim_job.simulator)
+    check_dut         (ip)
     
-    if sim_job.one_shot:
-        one_step_sim(sim_job)
-    else:
-        multi_step_sim(sim_job)
+    eal.init_workspace(sim_job)
+    
+    fsoc_core_name = ""
+    fsoc_core_flist_path = ""
+    if ip.dut_ip_type == "fsoc":
+        if sim_job.fsoc:
+            common.info(f"Processing FuseSoC Core '{ip.dut_core.name}' ...")
+            fsoc_core_flist_path = eal.invoke_fsoc(ip, ip.dut_core, sim_job)
+            if not sim_job.compile and not sim_job.elaborate and not sim_job.simulate:
+                common.info(f"Done.")
+        fsoc_core_name = ip.dut_core
+    elif ip.dut_ip_type != "":
+        if ip.dut_ip_type.target_ip_model.sub_type == "vivado":
+            dut_ip_str = f"{ip.dut.vendor}/{ip.dut.target_ip}"
+            common.info(f"Compiling Vivado Project DUT IP '{dut_ip_str}' ...")
+            est_time = get_ip_cmp_est(ip.dut_ip_type.target_ip_model, sim_job)
+            if est_time > 0:
+                pool = ThreadPool(processes=1)
+                pool.apply_async(progress_bar)
+                eal.cmp_vivado_project(ip.dut.target_ip_model, sim_job)
+                fill_progress_bar()
+                pool.terminate()
+                pool.join()
+            else:
+                eal.cmp_vivado_project(ip.dut.target_ip_model, sim_job)
+    
+    if sim_job.compile and sim_job.elaborate:
+        if sim_job.simulator == common.simulators_enum.VIVADO:
+            common.info(f"Compiling {ip_str} ...")
+            est_time = get_ip_cmp_est(ip, sim_job)
+            if est_time > 0:
+                pool = ThreadPool(processes=1)
+                pool.apply_async(progress_bar)
+                eal.cmp_ip(ip, sim_job, fsoc_core_name, fsoc_core_flist_path)
+                fill_progress_bar()
+                pool.terminate()
+                pool.join()
+            else:
+                eal.cmp_ip(ip, sim_job, fsoc_core_name, fsoc_core_flist_path)
+            common.info(f"Elaborating {ip_str} ...")
+            est_time = get_ip_elab_est(ip, sim_job)
+            if est_time > 0:
+                pool = ThreadPool(processes=1)
+                pool.apply_async(progress_bar)
+                eal.elab_ip(ip, sim_job)
+                fill_progress_bar()
+                pool.terminate()
+                pool.join()
+            else:
+                eal.elab_ip(ip, sim_job)
+        else:
+            common.info(f"Compiling+Elaborating {ip_str} ...")
+            est_time = get_ip_gen_image_est(ip, sim_job)
+            if est_time > 0:
+                pool = ThreadPool(processes=1)
+                pool.apply_async(progress_bar)
+                eal.gen_image_ip(ip, sim_job, fsoc_core_name, fsoc_core_flist_path)
+                fill_progress_bar()
+                pool.terminate()
+                pool.join()
+            else:
+                eal.gen_image_ip(ip, sim_job, fsoc_core_name, fsoc_core_flist_path)
+    elif sim_job.compile:
+        common.info(f"Compiling {ip_str} ...")
+        est_time = get_ip_cmp_est(ip, sim_job)
+        if est_time > 0:
+            pool = ThreadPool(processes=1)
+            pool.apply_async(progress_bar)
+            eal.cmp_ip(ip, sim_job, fsoc_core_name, fsoc_core_flist_path)
+            fill_progress_bar()
+            pool.terminate()
+            pool.join()
+        else:
+            eal.cmp_ip(ip, sim_job, fsoc_core_name, fsoc_core_flist_path)
+    elif sim_job.elaborate:
+        common.info(f"Elaborating {ip_str} ...")
+        est_time = get_ip_elab_est(ip, sim_job)
+        if est_time > 0:
+            pool = ThreadPool(processes=1)
+            pool.apply_async(progress_bar)
+            eal.elab_ip(ip, sim_job)
+            fill_progress_bar()
+            pool.terminate()
+            pool.join()
+        else:
+            eal.elab_ip(ip, sim_job)
+    
+    if not sim_job.is_regression:
+        if sim_job.simulate:
+            common.banner(f"Simulating {ip_str} ...")
+            if sim_job.gui and sim_job.simulator == common.simulators_enum.METRICS:
+                common.warning("The Metrics Cloud Simulator does not support GUI mode")
+            eal.simulate(ip, sim_job)
+    
+    if not sim_job.is_regression:
+        if sim_job.compile and sim_job.elaborate:
+            if sim_job.simulator == common.simulators_enum.VIVADO:
+                print_end_of_compilation_message(ip, sim_job)
+                print_end_of_elaboration_message(ip, sim_job)
+            else:
+                print_end_of_gen_image_message(ip, sim_job)
+        elif sim_job.compile:
+            print_end_of_compilation_message(ip, sim_job)
+        elif sim_job.elaborate:
+            print_end_of_elaboration_message(ip, sim_job)
+        if sim_job.simulate:
+            print_end_of_simulation_message(ip, sim_job)
     
     if sim_job.bwrap:
         bubble_wrap(sim_job)
 
 
-def one_step_sim(sim_job):
-    global est_time
-    sim_str = common.get_simulator_short_name(sim_job.simulator)
-    if sim_job.vendor == "":
-        ip = cache.get_anon_ip(sim_job.ip, True)
-    else:
-        ip = cache.get_ip(sim_job.vendor, sim_job.ip, True)
-    if ip == None:
-        common.fatal(f"Cannot find IP '{sim_job.vendor}/{sim_job.ip}'")
-    ip_str = f"{ip.vendor}/{ip.name}"
-    flist_path = ""
-    fsoc_core_name = ""
-    if ip.dut_ip_type == "fsoc":
-        if not ip.dut_core.is_installed:
-            flist_path = eal.invoke_fsoc(ip, ip.dut_core, sim_job)
-            ip.dut_core.is_installed = True
-            fsoc_core_name = ip.dut_core
-        else:
-            common.info("Skipping processing of DUT FuseSoC core '" + ip.dut_fsoc_full_name + "'.")
-    else:
-        if ip.dut != None:
-            if ip.dut.target_ip_model == None:
-                common.fatal(f"Did not resolve DUT dependency ('{ip.dut.vendor}/{ip.dut.target_ip}')!")
-            if ip.dut.target_ip_model.sub_type == "vivado":
-                cmp_dut(ip, sim_job)
-                #common.fatal("Vivado Project DUTs are not yet amenable to single-step compilation/elaboration flow.")
-    
-    common.info(f"Compiling+Elaborating {ip_str} ...")
-    eal.gen_ip_image(ip, sim_job, fsoc_core_name, flist_path)
-    if not sim_job.is_regression:
-        common.banner(f"Simulating {ip_str} ...")
-        eal.simulate(ip, sim_job)
-        print_end_of_simulation_message(ip, sim_job)
-
-
-def multi_step_sim(sim_job):
-    global est_time
-    sim_str = common.get_simulator_short_name(sim_job.simulator)
-    if sim_job.vendor == "":
-        ip = cache.get_anon_ip(sim_job.ip, True)
-    else:
-        ip = cache.get_ip(sim_job.vendor, sim_job.ip, True)
-    if ip == None:
-        common.fatal(f"Cannot find IP '{sim_job.vendor}/{sim_job.ip}'")
-    ip_str = f"{ip.vendor}/{ip.name}"
-    
-    cmp_ip_count = 0
-    if sim_job.compile:
-        if ip.has_dut:
-            compile_dut = True
-            if ip.dut_ip_type == "fsoc":
-                compile_dut = True #ip.dut_core.is_compiled[sim_job.simulator]
-            else:
-                compile_dut = True #ip.is_compiled[sim_job.simulator]
-            if compile_dut:
-                cmp_ip_count = cmp_ip_count + 1
-                if ip.dut_ip_type == "fsoc":
-                    dut_str = f"{ip.dut_fsoc_name}"
-                else:
-                    dut_str = f"{ip.dut.vendor}/{ip.dut.target_ip}"
-                if (not sim_job.dry_run) and (dut_str in cfg.job_history):
-                    est_time = 0
-                    curr_est_time = 0
-                    if 'compilation' in cfg.job_history[dut_str]:
-                        for job in cfg.job_history[dut_str]['compilation']:
-                            start = datetime.strptime(job['timestamp_start'], "%Y/%m/%d-%H:%M:%S")
-                            end   = datetime.strptime(job['timestamp_end'  ], "%Y/%m/%d-%H:%M:%S")
-                            curr_est_time = end - start
-                            curr_est_time = divmod(curr_est_time.seconds, 60)[1]
-                            est_time += curr_est_time
-                        est_time = math.ceil(est_time / len(cfg.job_history[dut_str]['compilation']))
-                        if est_time > 0:
-                            pool = ThreadPool(processes=1)
-                            pool.apply_async(progress_bar)
-                            cmp_dut(ip, sim_job)
-                            pool.terminate()
-                            pool.join()
-                        else:
-                            cmp_dut(ip, sim_job)
-                    else:
-                        cmp_dut(ip, sim_job)
-                else:
-                    cmp_dut(ip, sim_job)
-        
-        cmp_ip_count = cmp_ip_count + cmp_dependencies(ip, sim_job)
-        
-        #if not ip.is_compiled[sim_job.simulator]:
-        if True:
-            if not sim_job.is_regression:
-                common.banner("Compiling IP '" + ip_str + "'")
-            cmp_ip_count = cmp_ip_count + 1
-            if (not sim_job.dry_run) and (ip_str in cfg.job_history):
-                if 'compilation' in cfg.job_history[ip_str]:
-                    est_time = 0
-                    curr_est_time = 0
-                    for job in cfg.job_history[ip_str]['compilation']:
-                        start = datetime.strptime(job['timestamp_start'], "%Y/%m/%d-%H:%M:%S")
-                        end   = datetime.strptime(job['timestamp_end'  ], "%Y/%m/%d-%H:%M:%S")
-                        curr_est_time = end - start
-                        curr_est_time = divmod(curr_est_time.seconds, 60)[1]
-                        est_time += curr_est_time
-                    est_time = math.ceil(est_time / len(cfg.job_history[ip_str]['compilation']))
-                    if est_time > 0:
-                        #pool = ThreadPool(processes=1)
-                        #pool.apply_async(progress_bar)
-                        cmp_target_ip(ip, sim_job)
-                        #pool.terminate()
-                        #pool.join()
-                    else:
-                        cmp_target_ip(ip, sim_job)
-                else:
-                    cmp_target_ip(ip, sim_job)
-            else:
-                cmp_target_ip(ip, sim_job)
-    
-    needs_elaboration = True #not ip.is_elaborated[sim_job.simulator]
-    #if cmp_ip_count > 0:
-    #    needs_elaboration = True
-    
-    if sim_job.elaborate and needs_elaboration:
-        if sim_job.is_regression:
-            common.info("Elaborating IP '" + ip_str + "'")
-        else:
-            common.banner("Elaborating IP '" + ip_str + "'")
-        est_time = 0
-        curr_est_time = 0
-        if ip_str in cfg.job_history:
-            if (not sim_job.dry_run) and ('elaboration' in cfg.job_history[ip_str]):
-                for job in cfg.job_history[ip_str]['elaboration']:
-                    start = datetime.strptime(job['timestamp_start'], "%Y/%m/%d-%H:%M:%S")
-                    end   = datetime.strptime(job['timestamp_end'  ], "%Y/%m/%d-%H:%M:%S")
-                    curr_est_time = end - start
-                    curr_est_time = divmod(curr_est_time.seconds, 60)[1]
-                    est_time += curr_est_time
-                est_time = math.ceil(est_time / len(cfg.job_history[ip_str]['elaboration']))
-                if est_time > 0:
-                    #pool = ThreadPool(processes=1)
-                    #pool.apply_async(progress_bar)
-                    eal.elaborate(ip, sim_job)
-                    #pool.terminate()
-                    #pool.join()
-                else:
-                    eal.elaborate(ip, sim_job)
-            else:
-                eal.elaborate(ip, sim_job)
-        else:
-            eal.elaborate(ip, sim_job)
-    else:
-        if sim_job.simulate == False and not sim_job.is_regression:
-            print_end_of_compilation_message(ip, sim_job)
-    
-    if sim_job.simulate:
-        if sim_job.gui and sim_job.simulator == common.simulators_enum.METRICS:
-            common.warning("The Metrics Cloud Simulator does not support GUI mode")
-        if not sim_job.is_regression:
-            common.banner("Simulating IP '" + ip_str + "'")
-        eal.simulate(ip, sim_job)
-        if sim_job.compile == True and not sim_job.is_regression:
-            print_end_of_compilation_message(ip, sim_job)
-        if sim_job.elaborate == True and not sim_job.is_regression:
-            print_end_of_elaboration_message(ip, sim_job)
-        if not sim_job.is_regression:
-            print_end_of_simulation_message(ip, sim_job)
-    else:
-        if sim_job.elaborate == True and not sim_job.is_regression:
-            if sim_job.compile == True:
-                print_end_of_compilation_message(ip, sim_job)
-            print_end_of_elaboration_message(ip, sim_job)
-
-
-
-def bubble_wrap(sim_job):
-    sim_str = common.get_simulator_short_name(sim_job.simulator)
-    run_script_path = f"{cfg.sim_dir}/run.sh"
-    readme_path = f"{cfg.sim_dir}/README.txt"
-    
-    if sim_job.simulator == common.simulators_enum.VIVADO:
-        bin_text = "MIO_VIVADO_HOME"
-        bin_string = "/tools/vivado/bin"
-    if sim_job.simulator == common.simulators_enum.METRICS:
-        bin_text = "MIO_METRICS_HOME"
-        bin_string = "/usr/local/bin"
-    if sim_job.simulator == common.simulators_enum.VCS:
-        bin_text = "MIO_VCS_HOME"
-        bin_string = "/tools/vcs/bin"
-    if sim_job.simulator == common.simulators_enum.QUESTA:
-        bin_text = "MIO_QUESTA_HOME"
-        bin_string = "/tools/questa/bin"
-    if sim_job.simulator == common.simulators_enum.XCELIUM:
-        bin_text = "MIO_XCELIUM_HOME"
-        bin_string = "/tools/xcelium/bin"
-    if sim_job.simulator == common.simulators_enum.RIVIERA:
-        bin_text = "MIO_RIVIERA_HOME"
-        bin_string = "/tools/riviera/bin"
-    
-    try:
-        with open(run_script_path, 'w') as run_script_file:
-            run_script_file.write(f"export PROJECT_ROOT_DIR=$(pwd)/..\n\n")
-            for flist in sim_job.bwrap_flists:
-                run_script_file.write(f"export {flist}={sim_job.bwrap_flists[flist]}\n")
-            run_script_file.write("\n\n\n\n")
-            for cmd in sim_job.bwrap_commands:
-                cmd = cmd.replace(cfg.project_dir , "${PROJECT_ROOT_DIR}")
-                cmd = cmd.replace("-f .mio/"      , "-f ${PROJECT_ROOT_DIR}/.mio/")
-                cmd = cmd.replace(cfg.vivado_home , "${MIO_VIVADO_HOME}" )
-                cmd = cmd.replace(cfg.metrics_home, "${MIO_METRICS_HOME}")
-                cmd = cmd.replace(cfg.vcs_home    , "${MIO_VCS_HOME}"    )
-                cmd = cmd.replace(cfg.xcelium_home, "${MIO_XCELIUM_HOME}")
-                cmd = cmd.replace(cfg.questa_home , "${MIO_QUESTA_HOME}" )
-                cmd = cmd.replace(cfg.riviera_home, "${MIO_RIVIERA_HOME}")
-                run_script_file.write(f"{cmd}\n\n")
-        common.info(f"Wrote {run_script_path}")
-        with open(readme_path, 'w') as readme_file:
-            readme_file.write(f"1. Set ${bin_text}.  Ex: export {bin_text}={bin_string}\n")
-            readme_file.write(f"2. Run: bash ./run.sh\n")
-        common.info(f"Wrote {readme_path}")
-        
-        tarball_filename = f"{sim_job.ip}.{sim_job.test}.{sim_job.seed}.{sim_str}.tgz"
-        tarball_path = f"{cfg.project_dir}/../{tarball_filename}"
-        common.info(f"Writing {tarball_path} ...")
-        with tarfile.open(tarball_path, "w:gz") as tar:
-            files = os.listdir(cfg.project_dir)
-            for file in files:
-                file_path = f"{cfg.project_dir}/{file}"
-                if os.path.isdir(file_path):
-                    if file in bwrap_ignore_dirs:
-                        continue
-                common.dbg(f"Compressing {file}")
-                tar.add(file_path, filter=bwrap_exclude, arcname=file)
-        common.info("Done")
-        
-    except Exception as e:
-        common.fatal(f"Failed to create bubble-wrap tarball: {e}")
-
-
-
-def bwrap_exclude(tar_info):
-    for regex in bwrap_ignore_list:
-        if tar_info.name.endswith(regex):
-            return None
-    return tar_info
-
-
-
-def kill_progress_bar():
-    pass
-atexit.register(kill_progress_bar)
-
-
-
-def progress_bar():
-    global pbar
-    global seconds_waited
-    with tqdm(total=est_time) as pbar:
-        for i in range(est_time):
-            sleep(1)
-            pbar.update(1)
-    #with tqdm(total=seconds) as pbar:
-    #    for i in range(seconds):
-    #        sleep(1)
-    #        pbar.update(1)
-        
-    #with alive_bar(seconds, bar = 'smooth', stats="{eta} estimated", monitor=False, elapsed=True) as bar:
-    #    bar.text("estimated")
-    #    for x in range(seconds):
-    #        time.sleep(1)
-    #        seconds_waited += 1
-    #        bar()
-
-
-
 
 def convert_cli_args_to_defines(sim_job):
     defines = {}
@@ -523,7 +351,7 @@ def convert_cli_args_to_plusargs(sim_job):
     sim_job.sim_args = plus_args
 
 
-def create_sim_directories():
+def create_sim_directories(sim_job):
     common.dbg("Creating sim directories")
     common.create_dir(cfg.sim_dir)
     common.create_dir(cfg.sim_output_dir)
@@ -558,14 +386,13 @@ def create_sim_directories():
     common.create_dir(cfg.sim_output_dir + "/riv/regr_wd"             )
     common.create_dir(cfg.sim_output_dir + "/riv/so_libs"             )
     common.create_dir(cfg.sim_dir + "/cmp")
-    common.create_dir(cfg.sim_dir + "/elab")
     common.create_dir(cfg.regr_results_dir)
     common.create_dir(cfg.sim_results_dir)
 
 
 def check_dependencies(ip):
     for dep in ip.dependencies:
-        common.dbg(f"Checking dependency '{dep.vendor}/{dep.target_ip}'")
+        #common.dbg(f"Checking dependency '{dep.vendor}/{dep.target_ip}'")
         found_ip = cache.get_ip(dep.vendor, dep.target_ip)
         if found_ip == None:
             common.fatal(f"Could not find IP dependency '{dep.vendor}/{dep.target_ip}'")
@@ -573,112 +400,115 @@ def check_dependencies(ip):
             check_dependencies(found_ip)
 
 
-def cmp_dependencies(ip, sim_job):
-    global bar
-    global est_time
-    global num_deps_to_compile
-    deps = ip.get_ordered_deps()
-    deps_to_cmp = []
-    for dep in deps:
-        common.dbg(f"Processing dep {dep.vendor}/{dep.name} to be compiled")
-        if dep.name == "uvm":
-            continue
-        #if not dep.is_compiled[sim_job.simulator]:
-        deps_to_cmp.append(dep)
-    num_deps = len(deps_to_cmp)
-    if num_deps > 0:
-        if num_deps == 1:
-            msg = f"Compiling 1 dependency"
-        else:
-            msg = f"Compiling {num_deps} dependencies"
-        common.info(msg)
-    else:
-        return 0
-    
-    sim_str = common.get_simulator_short_name(sim_job.simulator)
-    defines = sim_job.cmp_args
-    est_time = num_deps
-    num_deps_to_compile = num_deps
-    with tqdm(deps_to_cmp) as pbar:
-        for dep in tqdm(deps_to_cmp):
-            dep_str = f"{dep.vendor}/{dep.name}"
-            pbar.set_description(dep_str)
-            eal.compile_ip(dep, sim_job)
-            pbar.update(1)
-    #with alive_bar(num_deps, bar = 'smooth', stats="{eta} estimated", monitor=True, elapsed=True) as bar:
-    #    for dep in deps_to_cmp:
-    #        dep_str = f"{dep.vendor}/{dep.name}"
-    #        bar.text(dep_str)
-    #        eal.compile_ip(dep, sim_job)
-    #        bar()
-    return num_deps_to_compile
-
-
-def cmp_dep(dep, sim_job):
-    global num_deps_to_compile
-    dep_str = f"{dep.vendor}/{dep.name}"
-    common.dbg(f"Compiling IP dependency '{dep_str}'")
-    eal.compile_ip(dep, sim_job)
-    sem.acquire()
-    bar.text(dep_str)
-    num_deps_to_compile = num_deps_to_compile - 1
-    sem.release()
-
-
-def cmp_dut(ip, sim_job):
-    defines = sim_job.cmp_args
-    
+def check_dut(ip):
     if ip.dut_ip_type == "fsoc":
-        if not ip.dut_core.is_installed or sim_job.fsoc:
-            flist_path = eal.invoke_fsoc(ip, ip.dut_core, sim_job)
-            ip.dut_core.is_installed = True
-        else:
-            common.info("Skipping processing of DUT FuseSoC core '" + ip.dut_fsoc_full_name + "'.")
-        eal.compile_fsoc_core(flist_path, ip.dut_core, sim_job)
+        pass
     else:
         if ip.dut != None:
             if ip.dut.target_ip_model == None:
                 common.fatal(f"Did not resolve DUT dependency ('{ip.dut.vendor}/{ip.dut.target_ip}')!")
-            dut_ip_str = f"{ip.dut.vendor}/{ip.dut.target_ip}"
-            compile_dut = False
-            if ip.dut.target_ip_model.is_local:
-                common.dbg("Found local IP DUT '" + dut_ip_str + "'")
-                compile_dut = True
-            else:
-                common.dbg("Found external IP DUT '" + dut_ip_str + "'")
-                if not ip.dut.target_ip_model.is_compiled[sim_job.simulator]:
-                    compile_dut = True
-            if compile_dut:
-                if ip.dut.target_ip_model.sub_type == "vivado":
-                    common.info("Compiling Vivado Project DUT IP '" + dut_ip_str + "'")
-                    eal.compile_vivado_project(ip.dut.target_ip_model, sim_job)
-                else:
-                    common.info("Compiling DUT IP '" + dut_ip_str + "'")
-                    eal.compile_ip(ip.dut.target_ip_model, sim_job)
-            else:
-                common.info("Skipping compilation of DUT IP '" + dut_ip_str + "'.")
-        else:
-            common.fatal("Could not find DUT IP '" + dut_ip_str + "'.")
 
 
-def cmp_target_ip(ip, sim_job):
-    defines = sim_job.cmp_args
-    ip_str = f"{ip.vendor}/{ip.name}"
-    if ip.sub_type == "vivado":
-        #common.info(f"Compiling Vivado Project '{ip_str}'")
-        eal.compile_vivado_project(ip_yml, sim_job)
+
+
+def bubble_wrap(sim_job):
+    sim_str = common.get_simulator_short_name(sim_job.simulator)
+    run_script_path = f"{cfg.sim_dir}/run.sh"
+    readme_path = f"{cfg.sim_dir}/README.txt"
+    
+    if sim_job.simulator == common.simulators_enum.VIVADO:
+        bin_text = "MIO_VIVADO_HOME"
+        bin_string = "/tools/vivado/bin"
+    if sim_job.simulator == common.simulators_enum.METRICS:
+        bin_text = "MIO_METRICS_HOME"
+        bin_string = "/usr/local/bin"
+    if sim_job.simulator == common.simulators_enum.VCS:
+        bin_text = "MIO_VCS_HOME"
+        bin_string = "/tools/vcs/bin"
+    if sim_job.simulator == common.simulators_enum.QUESTA:
+        bin_text = "MIO_QUESTA_HOME"
+        bin_string = "/tools/questa/bin"
+    if sim_job.simulator == common.simulators_enum.XCELIUM:
+        bin_text = "MIO_XCELIUM_HOME"
+        bin_string = "/tools/xcelium/bin"
+    if sim_job.simulator == common.simulators_enum.RIVIERA:
+        bin_text = "MIO_RIVIERA_HOME"
+        bin_string = "/tools/riviera/bin"
+    
+    try:
+        with open(run_script_path, 'w') as run_script_file:
+            run_script_file.write(f"export PROJECT_ROOT_DIR=$(pwd)/..\n\n")
+            for flist in sim_job.bwrap_flists:
+                run_script_file.write(f"export {flist}={sim_job.bwrap_flists[flist]}\n")
+            run_script_file.write("\n\n\n\n")
+            for cmd in sim_job.bwrap_commands:
+                cmd = cmd.replace(cfg.project_dir , "${PROJECT_ROOT_DIR}")
+                cmd = cmd.replace("-f .mio/"      , "-f ${PROJECT_ROOT_DIR}/.mio/")
+                cmd = cmd.replace(cfg.vivado_home , "${MIO_VIVADO_HOME}" )
+                cmd = cmd.replace(cfg.metrics_home, "${MIO_METRICS_HOME}")
+                cmd = cmd.replace(cfg.vcs_home    , "${MIO_VCS_HOME}"    )
+                cmd = cmd.replace(cfg.xcelium_home, "${MIO_XCELIUM_HOME}")
+                cmd = cmd.replace(cfg.questa_home , "${MIO_QUESTA_HOME}" )
+                cmd = cmd.replace(cfg.riviera_home, "${MIO_RIVIERA_HOME}")
+                run_script_file.write(f"{cmd}\n\n")
+        common.info(f"Wrote {run_script_path}")
+        with open(readme_path, 'w') as readme_file:
+            readme_file.write(f"1. Set ${bin_text}.  Ex: export {bin_text}={bin_string}\n")
+            readme_file.write(f"2. Run: bash ./run.sh\n")
+        common.info(f"Wrote {readme_path}")
+        
+        tarball_filename = f"{sim_job.ip}.{sim_job.test}.{sim_job.seed}.{sim_str}.tgz"
+        tarball_path = f"{cfg.project_dir}/../{tarball_filename}"
+        common.info(f"Writing {tarball_path} ...")
+        with tarfile.open(tarball_path, "w:gz") as tar:
+            files = os.listdir(cfg.project_dir)
+            for file in files:
+                file_path = f"{cfg.project_dir}/{file}"
+                if os.path.isdir(file_path):
+                    if file in bwrap_ignore_dirs:
+                        continue
+                common.dbg(f"Compressing {file}")
+                tar.add(file_path, filter=bwrap_exclude, arcname=file)
+        common.info("Done")
+        
+    except Exception as e:
+        common.fatal(f"Failed to create bubble-wrap tarball: {e}")
+
+
+
+def bwrap_exclude(tar_info):
+    for regex in bwrap_ignore_list:
+        if tar_info.name.endswith(regex):
+            return None
+    return tar_info
+
+
+
+
+def print_end_of_gen_image_message(ip, sim_job):
+    if sim_job.dry_run:
+        return
+    ip_str = f"{ip.name}"
+    sim_str = common.get_simulator_short_name(sim_job.simulator)
+    if sim_job.simulate:
+        common.info("************************************************************************************************************************")
+        common.info("* Compilation+Elaboration results: " + cfg.sim_dir + "/cmp/" + ip_str + "." + sim_str + ".log")
     else:
-        #common.info(f"Compiling '{ip_str}'")
-        eal.compile_ip(ip, sim_job)
-
+        common.info("************************************************************************************************************************")
+        common.info("* Compilation+Elaboration results:")
+        common.info("************************************************************************************************************************")
+        common.info("  emacs " + cfg.sim_dir + "/cmp/" + ip_str + "." + sim_str + ".log &")
+        common.info("  gvim  " + cfg.sim_dir + "/cmp/" + ip_str + "." + sim_str + ".log &")
+        common.info("  vim   " + cfg.sim_dir + "/cmp/" + ip_str + "." + sim_str + ".log")
+        common.info("")
 
 
 def print_end_of_compilation_message(ip, sim_job):
     if sim_job.dry_run:
         return
-    ip_str = f"{ip.vendor}__{ip.name}"
+    ip_str = f"{ip.name}"
     sim_str = common.get_simulator_short_name(sim_job.simulator)
-    if sim_job.elaborate:
+    if sim_job.simulate or sim_job.elaborate:
         common.info("************************************************************************************************************************")
         common.info("* Compilation results: " + cfg.sim_dir + "/cmp/" + ip_str + "." + sim_str + ".cmp.log")
     else:
@@ -694,18 +524,18 @@ def print_end_of_compilation_message(ip, sim_job):
 def print_end_of_elaboration_message(ip, sim_job):
     if sim_job.dry_run:
         return
-    ip_str = f"{ip.vendor}__{ip.name}"
+    ip_str = f"{ip.name}"
     sim_str = common.get_simulator_short_name(sim_job.simulator)
     if sim_job.simulate:
         common.info("************************************************************************************************************************")
-        common.info("* Elaboration results: " + cfg.sim_dir + "/elab/" + ip_str + "." + sim_str + ".elab.log")
+        common.info("* Elaboration results: " + cfg.sim_dir + "/cmp/" + ip_str + "." + sim_str + ".elab.log")
     else:
         common.info("************************************************************************************************************************")
         common.info("* Elaboration results")
         common.info("************************************************************************************************************************")
-        common.info("  emacs " + cfg.sim_dir + "/elab/" + ip_str + "." + sim_str + ".elab.log &")
-        common.info("  gvim  " + cfg.sim_dir + "/elab/" + ip_str + "." + sim_str + ".elab.log &")
-        common.info("  vim   " + cfg.sim_dir + "/elab/" + ip_str + "." + sim_str + ".elab.log")
+        common.info("  emacs " + cfg.sim_dir + "/cmp/" + ip_str + "." + sim_str + ".elab.log &")
+        common.info("  gvim  " + cfg.sim_dir + "/cmp/" + ip_str + "." + sim_str + ".elab.log &")
+        common.info("  vim   " + cfg.sim_dir + "/cmp/" + ip_str + "." + sim_str + ".elab.log")
         common.info("")
 
 
@@ -736,8 +566,85 @@ def print_end_of_simulation_message(ip, sim_job):
     common.info("            gvim  " + results_path + "/sim.log &")
     common.info("            vim   " + results_path + "/sim.log")
     common.info("")
-    common.info("* Transaction logs: pushd " + results_path + "/trn_log")
     common.info("* Test result dir : pushd " + results_path)
+    common.info("* UVMx logs       : pushd " + results_path + "/uvmx_logs")
     common.info("")
 
 
+
+def get_ip_cmp_est(ip, sim_job):
+    ip_str = f"{ip.vendor}/{ip.name}"
+    sim_str = common.get_simulator_short_name(sim_job.simulator)
+    est_time = 0
+    if ip_str in cfg.job_history:
+        if 'compilation' in cfg.job_history[ip_str]:
+            for job in cfg.job_history[ip_str]['compilation']:
+                duration = job['duration']
+                est_time += int(duration)
+            est_time = math.ceil(est_time / len(cfg.job_history[ip_str]['compilation']))
+    return est_time
+
+
+
+def get_ip_elab_est(ip, sim_job):
+    ip_str = f"{ip.vendor}/{ip.name}"
+    sim_str = common.get_simulator_short_name(sim_job.simulator)
+    est_time = 0
+    if ip_str in cfg.job_history:
+        if 'elaboration' in cfg.job_history[ip_str]:
+            for job in cfg.job_history[ip_str]['elaboration']:
+                duration = job['duration']
+                est_time += int(duration)
+            est_time = math.ceil(est_time / len(cfg.job_history[ip_str]['elaboration']))
+    return est_time
+
+
+
+def get_ip_gen_image_est(ip, sim_job):
+    ip_str = f"{ip.vendor}/{ip.name}"
+    sim_str = common.get_simulator_short_name(sim_job.simulator)
+    est_time = 0
+    if ip_str in cfg.job_history:
+        if 'gen-image' in cfg.job_history[ip_str]:
+            for job in cfg.job_history[ip_str]['gen-image']:
+                duration = job['duration']
+                est_time += int(duration)
+            est_time = math.ceil(est_time / len(cfg.job_history[ip_str]['gen-image']))
+    return est_time
+
+
+
+def kill_progress_bar():
+    global pbar
+    if pbar:
+        pbar.close()
+atexit.register(kill_progress_bar)
+
+
+
+def fill_progress_bar():
+    global pbar
+    global seconds_waited
+    global est_time
+    global overtime
+    if pbar:
+        pbar.update(est_time-seconds_waited)
+        pbar.close()
+        overtime = False
+
+
+
+def progress_bar():
+    global pbar
+    global seconds_waited
+    global est_time
+    global overtime
+    with tqdm(total=est_time, bar_format="{l_bar} {bar} {elapsed}<{remaining}") as pbar:
+        for seconds_waited in range(est_time):
+            time.sleep(1)
+            pbar.update(1)
+    if overtime:
+        common.info(f"Job taking longer than average ({est_time} seconds)")
+
+
+
